package main

import (
	"fmt"
	"os/exec"
	"strconv"
	"strings"
	"time"

	"gogs.bullercodeworks.com/brian/termbox-util"

	"github.com/nsf/termbox-go"
)

type mainScreen struct {
	initialized bool
	tabIdx      int
	updateTime  time.Time
	battStat    *termboxUtil.ProgressBar
}

func (screen *mainScreen) handleKeyEvent(event termbox.Event) int {
	if event.Type == termbox.EventKey {
		if event.Key == termbox.KeyEsc {
			return exitScreenIndex
		} else if event.Ch == 'R' {
			state.autoUpdate = true
		} else if event.Ch == '+' {
			screen.decreaseUpdateFreq()
		} else if event.Ch == '-' {
			screen.increaseUpdateFreq()
		} else if event.Ch == 'r' {
			// Refresh data
			screen.refreshData()
		} else if event.Ch == 'q' {
			return exitScreenIndex
		}
	}
	return mainScreenIndex
}

func (screen *mainScreen) decreaseUpdateFreq() {
	if state.updateFreq > 60 {
		state.updateFreq += 60
	} else {
		state.updateFreq += 5
	}
}
func (screen *mainScreen) increaseUpdateFreq() {
	if state.updateFreq > 60 {
		state.updateFreq -= 60
	} else {
		state.updateFreq -= 5
	}
}

func (screen *mainScreen) refreshData() {
	screen.updateTime = time.Now()
	screen.battStat.SetProgress(getBatteryPct())
}

func (screen *mainScreen) performLayout(style style) {
	if !screen.initialized {
		w, _ := termbox.Size()
		screen.battStat = termboxUtil.CreateProgressBar(100, w-14, 1, style.defaultFg, style.defaultBg)
		screen.battStat.SetColorized(true)
		screen.refreshData()
	}
	if time.Since(time.Now()) <= time.Minute {
		screen.refreshData()
	}
}

func (screen *mainScreen) drawScreen(style style) {
	w, h := termbox.Size()
	termboxUtil.DrawStringAtPoint(screen.updateTime.Format("15:04:05 MST"), 0, 0, style.defaultFg, style.defaultBg)
	termboxUtil.DrawStringAtPoint(screen.updateTime.Format("02 Jan 2006"), 0, 1, style.defaultFg, style.defaultBg)
	screen.battStat.Draw()
	termboxUtil.DrawStringAtPoint(fmt.Sprintf("% 3d%s", screen.battStat.GetPercent(), "%"),
		screen.battStat.GetX()+screen.battStat.GetWidth(),
		screen.battStat.GetY(), style.defaultFg, style.defaultBg)

	termboxUtil.DrawStringAtPoint(getNetworkProfile(), 0, h-1, style.defaultFg, style.defaultBg)

	refreshText := "(r)efresh"
	if state.autoUpdate {
		refreshText = "Auto-Refresh On ("
		if state.updateFreq >= 60 {
			refreshText += strconv.Itoa(int(state.updateFreq/time.Minute)) + "m"
		} else {
			refreshText += strconv.Itoa(int(state.updateFreq/time.Second)) + "s"
		}
		refreshText += ")"
	}
	helpTxt := "(q)uit  " + refreshText
	termboxUtil.DrawStringAtPoint(helpTxt, (w - len(helpTxt)), h-1, style.defaultFg, style.defaultBg)
}

func getBatteryPct() int {
	powerApp := "upower"
	powerArg1 := "-i"
	powerArg2 := "/org/freedesktop/UPower/devices/battery_BAT0"
	cmd := exec.Command(powerApp, powerArg1, powerArg2)
	stdout, err := cmd.Output()
	if err != nil {
		return -1
	}
	tst := string(stdout)
	for _, k := range strings.Split(tst, "\n") {
		pair := strings.Split(k, ":")
		if len(pair) > 1 {
			pair[0] = strings.Trim(pair[0], " ")
			pair[1] = strings.Trim(pair[1], " %")
			if pair[0] == "percentage" {
				if pct, err := strconv.Atoi(pair[1]); err == nil {
					return pct
				}
			}
		}
	}
	return -1
}

func getNetworkProfile() string {
	netApp := "netctl"
	netArg1 := "list"
	cmd := exec.Command(netApp, netArg1)
	stdout, err := cmd.Output()
	if err == nil {
		tst := string(stdout)
		for _, k := range strings.Split(tst, "\n") {
			pair := strings.Split(k, " ")
			if len(pair) > 1 {
				if pair[0] == "*" {
					return pair[1]
				}
			}
		}
	}
	return "No Network"
}
